# Simple trackball-like motion adapted (ripped off) from projtex.c
# (written by David Yu and David Blythe).  See the SIGGRAPH '96
# Advanced OpenGL course notes.
#
# Usage:
#
#  o  call tcl3dTbInit in before any other trackball call
#  o  call tcl3dTbReshape from the reshape callback
#  o  call tcl3dTbMatrix to get the trackball matrix rotation
#  o  call tcl3dTbStartMotion to begin trackball movement
#  o  call tcl3dTbStopMotion to stop trackball movement
#  o  call tcl3dTbMotion from the motion callback
#  o  call tcl3dTbAnimate(1) if you want the trackball to continue 
#     spinning after the mouse button has been released
#  o  call tcl3dTbAnimate(0) if you want the trackball to stop 
#     spinning after the mouse button has been released
#
# Modified for Tcl3D by Paul Obermeier 2006/02/02
# See www.tcl3d.org for the Tcl3D extension.

proc _getTimeMS {} {
    global __tcl3dTB
    return [expr int ([tcl3dLookupSwatch $__tcl3dTB(swatch)] * 10)]
}

proc _stopIdleFunc {} {
    global __tcl3dTB
    if { [info exists __tcl3dTB(afterId)] } {
        after cancel $__tcl3dTB(afterId)
    }
}

proc _tbAnimate { toglwin } {
    global __tcl3dTB

    tcl3dTrackballAddQuats $__tcl3dTB(lastQuat) $__tcl3dTB(curQuat) $__tcl3dTB(curQuat)
    $toglwin postredisplay
    set __tcl3dTB(afterId) [after idle "_tbAnimate $toglwin"]
}

proc tcl3dTbStartMotion { toglwin x y } {
    global __tcl3dTB

    _stopIdleFunc
    set __tcl3dTB(tracking) 1
    set __tcl3dTB(lasttime) [_getTimeMS]
    set __tcl3dTB(beginx) $x
    set __tcl3dTB(beginy) $y
}

proc tcl3dTbStopMotion { toglwin } {
    global __tcl3dTB

    set __tcl3dTB(tracking) 0

    set t [_getTimeMS]
    if { ($t == $__tcl3dTB(lasttime)) && \
         $__tcl3dTB(animate) } {
        _tbAnimate $toglwin
    } else {
        _stopIdleFunc
    }
}

proc tcl3dTbAnimate { animate } {
    global __tcl3dTB

    set __tcl3dTB(animate) $animate
}

proc tcl3dTbInit {} {
    global __tcl3dTB

    set __tcl3dTB(swatch) [tcl3dNewSwatch]
    tcl3dStartSwatch $__tcl3dTB(swatch)
    set __tcl3dTB(lasttime) [_getTimeMS]
    set __tcl3dTB(tracking) 0
    set __tcl3dTB(animate)  1
    set __tcl3dTB(curQuat)  [tcl3dVector GLfloat 4]
    set __tcl3dTB(lastQuat) [tcl3dVector GLfloat 4]
    tcl3dTrackball $__tcl3dTB(curQuat) 0.0 0.0 0.0 0.0
}

proc tcl3dTbMatrix {} {
    global __tcl3dTB

    set m [tcl3dVector GLfloat 16]
    tcl3dTrackballBuildRotMatrix $m $__tcl3dTB(curQuat)
    set ml [tcl3dVectorToList $m 16]
    glMultMatrixf $ml
    $m delete
}

proc tcl3dTbReshape { toglwin w h } {
    global __tcl3dTB

    set __tcl3dTB(width)  $w
    set __tcl3dTB(height) $h
}

proc tcl3dTbMotion { toglwin x y } {
    global __tcl3dTB

    if { $__tcl3dTB(tracking) } {
        set __tcl3dTB(lasttime) [_getTimeMS]
        tcl3dTrackball $__tcl3dTB(lastQuat) \
        [expr (2.0 * $__tcl3dTB(beginx) - $__tcl3dTB(width)) / $__tcl3dTB(width)] \
        [expr ($__tcl3dTB(height) - 2.0 * $__tcl3dTB(beginy)) / $__tcl3dTB(height)] \
        [expr (2.0 * $x - $__tcl3dTB(width)) / $__tcl3dTB(width)] \
        [expr ($__tcl3dTB(height) - 2.0 * $y) / $__tcl3dTB(height)]
        set __tcl3dTB(beginx) $x
        set __tcl3dTB(beginy) $y
        set __tcl3dTB(animate) 1
        tcl3dTrackballAddQuats $__tcl3dTB(lastQuat) $__tcl3dTB(curQuat) $__tcl3dTB(curQuat)
        $toglwin postredisplay
    }
}
